                                                          (* 20.3.1997    *)
(*
   Satz von Morley: Drittelt man die Winkeln eines beliebigen Dreiecks und
   schneidet die jeweils einer Seite naheliegenden Winkeldreiteilungs-
   geraden, so entsteht immer ein gleichseitiges Dreieck.
*)

PROGRAM Morley;
USES CRT,GRAPH,UNIGRA12;

CONST
Eps = 1.0E-12;

TYPE
Line2D = RECORD                                                     
             O: Pt2D;
             U: Vt2D;
           END;

VAR
T1,T2       : EXTENDED;                  (* Parameter der Geraden =       *)
                                         (* Loesungen der quadr. Gleich.  *)
                                      (* Dreieckspunkte                   *)
A,B,C,S,NA,NB,NC          : Pt2D;   

AC,AB,BC                    : Line2D;                   (* Dreiecksseiten *)
Intsec                      : Boolean;
Alpha,Beta,Gamma            : EXTENDED;
AC3,CA3,AB3,BA3,BC3,CB3     : Line2D;
H1,H2 : Vt2D;
Alpha3,Beta3,Gamma3,W,W1    : EXTENDED;
sa,sb,sc                    : EXTENDED;

{ ------------------------- Unterprogramme ------------------------------ }
Procedure Ausgangszustand_definieren;
   BEGIN
   IF PLOT THEN Plotter_einschalten;
   WinitGraph(0,0,'');
   Weltgrenzen_festlegen(einh);
   Viewport_festlegen;

   IF PLOT THEN GeraetefensterPlotter_festlegen;
   Parameter2D;
   IF PLOT THEN Portrait  := FALSE;        (* Querformat Bild liegend      *)
   IF PLOT THEN WRITE(AUX,'WU0');          (* width unit absolut in mm     *)
   Plast.X := WI2D[1].X;                 (* letzter Punkt initialisieren   *)
   Plast.Y := WI2D[1].Y;

   Viewport_zeichnen; 
   Koordinatenachsen_zeichnen;
   Gitter(0.3,2.0,10.0);(* Strich,              klE,             grE     *)
                       (* grosse Strichlaenge,kleine Einheit, grosse Einh *)
  END;

Procedure Beschriftung;
  VAR
  T: Ttyp;
  XUntergr,XObergr,YUntergr,YObergr,einhS: STRING[Bz1];
  Ttemp1,Ttemp2: STRING[Bz2];

  AXS,AYS,BXS,BYS,CXS,CYS              : STRING[Bz1];
  saS,sbS,scS                          : STRING[Bz1];
  BEGIN
  STR(WI2D[1].X:5:1,XUntergr); STR(WI2D[1].Y:5:1,YUntergr);
  STR(WI2D[2].X:5:1,XObergr);  STR(WI2D[2].Y:5:1,YObergr);
  STR(Einh:5:3,EinhS);
  
  STR(A.X:5:2,AXS);  STR(A.Y:5:2,AYS);
  STR(B.X:5:2,BXS);  STR(B.Y:5:2,BYS);
  STR(C.X:5:2,CXS);  STR(C.Y:5:2,CYS);
  STR(sa:6:4,saS);   STR(sb:6:4,sbS);    STR(sc:6:4,scS);

(* ----------------------------- TEXTE --------------------------------- *)
  
  Ttemp1 := 'geg: Dreieck A('+AXS+'/'+AYS+')   B('+BXS+'/'+BYS+')   ';
  Ttemp2 := 'C('+CXS+'/'+CYS+')';
  T[1] := Ttemp1 + Ttemp2;
  T[2] := 'Satz von Morley ergibt ein gleichseitiges Dreieck:';
  Ttemp1 := 'Seite a = '+saS+'   Seite b = '+sbS ;
  Ttemp2 := '      Seite c = '+scS;
  T[3] := Ttemp1 + Ttemp2;
        Ttemp1 := 'Weltgrenzen: X von '+XUntergr+' bis '+XObergr;
        Ttemp2 := '   Y von '+YUntergr+' bis '+YObergr;
  T[4] := Ttemp1 + Ttemp2;
  T[5] := '1 Einheit entspricht '+EinhS+'  cm';
  Text_ausgeben(T,5);                                 (* .. Texte ausgeben *)
  END;

PROCEDURE MidPoint(P1,P2: Pt2D; VAR M: Pt2D);
   BEGIN
     M.X := (P1.X+P2.X)/2.0;
     M.Y := (P1.Y+P2.Y)/2.0;
   END;

PROCEDURE MakeVt2D(P1,P2: Pt2D; VAR V: Vt2D);
  BEGIN
   V.X := P2.X - P1.X;
   V.Y := P2.Y - P1.Y;
  END;

PROCEDURE LengthVt2D(V: Vt2D; VAR L: EXTENDED);
BEGIN
  L := SQRT(SQR(V.X)+SQR(V.Y)+Eps);
END;

PROCEDURE ScaleVt2D(V: Vt2D; LG: EXTENDED; VAR ScalV: Vt2D);
VAR L: EXTENDED;
BEGIN
  ScalV:=V;
  LengthVt2D(V, L);
  IF NOT Null(L,Eps) THEN BEGIN
    ScalV.X:=V.X/L*LG;
    ScalV.Y:=V.Y/L*LG;
  END;
END;

PROCEDURE CosSinToAngle(C,S: EXTENDED; VAR Ang: EXTENDED);
BEGIN
   IF Null(C,Eps) AND (S<0.0) THEN Ang := -PI/2.0;
   IF Null(C,Eps) AND (S>=0.0) THEN Ang := PI/2.0;
   IF NOT Null(C,Eps) THEN Ang := ARCTAN(S/C);
   IF (C<0) AND (S<0.0) THEN Ang := Ang-PI;
   IF (C<0) AND (S>=0.0) THEN Ang := Ang+PI;
END;

PROCEDURE AngleBetweenVts2D (V1,V2: Vt2D; VAR Ang:EXTENDED);
VAR C,S: EXTENDED;
BEGIN
   C:=V1.X*V2.X + V1.Y*V2.Y;
   S:=V1.X*V2.Y - V1.Y*V2.X;
   CosSinToAngle(C,S, Ang);
END;

PROCEDURE MakeLine2D(P1,P2: Pt2D; VAR Line: Line2D);
  VAR
  hv : Vt2D;
  BEGIN
    MakeVt2D(P1,P2,hv);
    ScaleVt2D(hv,1.0,hv);
    Line.O := P1;
    Line.U := hv;
  END;

PROCEDURE SumVt2d(V1,V2:Vt2D; VAR V: Vt2D);
  BEGIN
   V.X := V1.X+V2.X;
   V.Y := V1.Y+V2.Y;
  END;

PROCEDURE DifferenceVt2D(V1,V2: Vt2D; VAR DV: Vt2D);
BEGIN
  DV.X := V1.X-V2.X;
  DV.Y := V1.Y-V2.Y;
END;

PROCEDURE LinearCombinationVt2D(S1,S2: EXTENDED; V1,V2: Vt2D; VAR LC: Vt2D);
BEGIN
  LC.X:=S1*V1.X+S2*V2.X;
  LC.Y:=S1*V1.Y+S2*V2.Y;
END;

PROCEDURE ScalarProductVt2D(V1,V2: Vt2D; VAR SP: EXTENDED);
BEGIN
  SP:=V1.X*V2.X+V1.Y*V2.Y;
END;

PROCEDURE TToLine2D(Ln2D: Line2D; T: EXTENDED; VAR P: Pt2D);
BEGIN
  LinearCombinationVt2D(1,T,Ln2D.O,Ln2D.U, P);
END;

PROCEDURE IntersectionLines2D(Line1,Line2: Line2D; VAR Intsec: BOOLEAN;
                              VAR T1,T2: EXTENDED; VAR S: Pt2D);
VAR Num,Den: EXTENDED;
    N2,DV: Vt2D;
    V: Vt2D;
BEGIN
  Intsec := FALSE;
  N2.X := Line2.U.Y; N2.Y := -Line2.U.X;
  DifferenceVt2D(Line1.O, Line2.O, DV);
  ScalarProductVt2D(DV,N2, Num);
  ScalarProductVt2D(Line1.U,N2, Den);
  IF NOT Null(Den,Eps) THEN BEGIN
    T1 := -Num/Den;
    TToLine2D(Line1,T1, S);
    DifferenceVt2D(S,Line2.O, V);       (* nicht mehr notwendig, nur mehr *)
    ScalarProductVt2D(V,Line2.U, T2);   (* zur Berechnung von t2          *)
    Intsec := TRUE;
  END;
END;

PROCEDURE NormLinie;
  BEGIN
    IF PLOT THEN BEGIN
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0.15');          (* 0.15 mm                    *)
               WRITE(AUX,'UL');              (* 6.:                        *)
               WRITE(AUX,'LT');              (* solid                      *)
    END;
  SetLineStyle(0,0,1);
  END;

PROCEDURE Hilfslinie1;
  BEGIN
    IF PLOT THEN BEGIN
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0');              (* ganz schwache Hilfslinie: *)
               WRITE(AUX,'UL8,1,9,1,9,1,9');  (* 1.:                       *)
               WRITE(AUX,'LT8,4.0,1');       
    END;
    SetLineStyle(4,$8080,1);
  END;

PROCEDURE HilfsLinie2;
  BEGIN
    IF PLOT THEN BEGIN
          {    WRITE(AUX,'PW0.3');       }    (* staerkere Hilfslinie      *)
          {    WRITE(AUX,'LT1,0.8,1');   }    (* hellgrau length 1.0 mm    *)
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0');
               WRITE(AUX,'UL8,2,7,2,7,2,7');
               WRITE(AUX,'LT8,2.0,1');      
    END;
    SetLineStyle(1,0,1);
  END;

PROCEDURE WeitStrichliert;
  BEGIN
    IF PLOT THEN BEGIN
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0');             (* duennste Linie strichliert *)
               WRITE(AUX,'UL8,7,4,7,4,7,4'); (* 3.:                        *)
               WRITE(AUX,'LT8,7.0,1');       (*  weite Abstaende           *)
    END;
    SetLineStyle(4,$FF00,1);
  END;

PROCEDURE EngStrichliert;
  BEGIN
    IF PLOT THEN BEGIN
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0');             (* duennste Linie strichliert *)
               WRITE(AUX,'UL8,7,2,7,2,7,2'); (* 5.:                        *)
               WRITE(AUX,'LT8,10.0,1');
    END;
    SetLineStyle(3,0,1);
  END;

PROCEDURE StrichPunktiert;                   (* als Symmetrieachse         *)
  BEGIN
    IF PLOT THEN BEGIN
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0');             (* duennste Linie             *)
               WRITE(AUX,'UL8,8,2,2,2,8,2,2,2,8,2,2,2');
               WRITE(AUX,'LT8,15.0,1');      (* strichpunktiert mit klein. *)
    END;                                     (* Strich als Punkt           *)
    SetLineStyle(4,$F18F,1);                  
  END;                                           

PROCEDURE BreitLinie;
  BEGIN
    IF PLOT THEN BEGIN
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0.3');           (* 0.3 durchgehend            *)
               WRITE(AUX,'UL');              (* 7.:                        *)
               WRITE(AUX,'LT');
    END;
    SetLineStyle(0,0,1);
  END;
   
(* -------------------- eigentliche Routinen ----------------------------- *)

PROCEDURE Eingabe;                
  BEGIN                         
   A.X := -14.6;  A.Y := - 3.5;                          (* Dreieckspunkte *)
   B.X :=  32.3;  B.Y := -19.6;
   C.X :=   5.8;  C.Y :=  18.3;
  END;

PROCEDURE Dreiecks_Seiten;
  BEGIN
    MakeLine2D(A,B,AB);
    MakeLine2D(A,C,AC);
    MakeLine2D(B,C,BC);
  END;

PROCEDURE Winkeln;
  VAR
  i : INTEGER;
  BEGIN
    AngleBetweenVts2D(AB.U,AC.U,Alpha);   (* Reihenfolge der Vektoren    *)
    H1.X := -AB.U.X; H1.Y := -AB.U.Y;     (* gegen den Uhrzeigersinn     *)
    AngleBetweenVts2D(BC.U,H1,Beta);      (* dann positive Winkelwerte   *)
    H1.X := -AC.U.X; H1.Y := -AC.U.Y;
    H2.X := -BC.U.X; H2.Y := -BC.U.Y;
    AngleBetweenVts2D(H1,H2,Gamma);
    Alpha3 := Alpha/3.0;                   (* Winkeln dritteln            *)
    Beta3 := Beta/3.0;
    Gamma3 := Gamma/3.0;
    (* ----------------- *)
    H1.X := 1.0; H1.Y := 0.0;                                 (* X-Achse *)
    AngleBetweenVts2D(H1,AC.U,W1);
    W := W1 - Alpha3;
    AC3.U.X := COS(W); AC3.U.Y := SIN(W);   AC3.O := A;

    H2.X := -AC.U.X; H2.Y := -AC.U.Y;
    AngleBetweenVts2D(H2,H1,W1);
    W := 2.0*Pi-(W1 - Gamma3);
    CA3.U.X := COS(W); CA3.U.Y := SIN(W); CA3.O := C;
    IntersectionLines2D(AC3,CA3,IntSec,T1,T2,S);
    NA := S;

    
    H1.X := 1.0; H1.Y := 0.0;                                 (* X-Achse *)
    AngleBetweenVts2D(AB.U,H1,W1);
    W := 2.0*Pi-(W1 - Alpha3);
    AB3.U.X := COS(W); AB3.U.Y := SIN(W); AB3.O := A;
                                                       
    H2.X := -AB.U.X; H2.Y := -AB.U.Y;
    AngleBetweenVts2D(H1,H2,W1);
    W := W1 - Beta3;
    BA3.U.X := COS(W); BA3.U.Y := SIN(W); BA3.O := B;
    IntersectionLines2D(BA3,AB3,IntSec,T1,T2,S);  
    NB := S;

                                                            
    H1.X := 1.0; H1.Y := 0.0;                                 (* X-Achse *)
    AngleBetweenVts2D(H1,BC.U,W1);
    W := W1 + Beta3;
    BC3.U.X := COS(W); BC3.U.Y := SIN(W); BC3.O := B;

    H2.X := -BC.U.X; H2.Y := -BC.U.Y;
    AngleBetweenVts2D(H2,H1,W1);
    W := 2.0*Pi-(W1 + Gamma3);
    CB3.U.X := COS(W); CB3.U.Y := SIN(W); CB3.O := C;
    IntersectionLines2D(BC3,CB3,IntSec,T1,T2,S);  
    NC := S;
  END;

PROCEDURE Seitenlaengen;
  VAR
  H: Vt2D;
  BEGIN
    MakeVt2D(NA,NB,H);
    LengthVt2D(H,sc);
    MakeVt2D(NA,NC,H);
    LengthVt2D(H,sb);
    MakeVt2D(NB,NC,H);
    LengthVt2D(H,sa);
  END;



PROCEDURE Zeichnen;

  BEGIN
  IF PLOT THEN WRITE(AUX,'WU0');                                            
  NormLinie;
  DrawLine2D(A,B,15);                         (* Die 3 Dreiecksseiten:    *)
  DrawLine2D(A,C,15);
  DrawLine2D(B,C,15);
  
  Hilfslinie2;
  DrawLine2D(A,NA,7);
  DrawLine2D(C,NA,7);
  DrawLine2D(A,NB,7);
  Hilfslinie2;
  DrawLine2D(B,NB,7);
  Hilfslinie2;
  DrawLine2D(B,NC,7);
  Hilfslinie2;
  DrawLine2D(C,NC,7);

  NormLinie;
  DrawLine2D(NA,NB,14);
  NormLinie;
  DrawLine2D(NA,NC,14);
  NormLinie;
  DrawLine2D(NB,NC,14);
END;

{ --------------------------- Hauptprogramm ------------------------------- }
BEGIN
  Plot := FALSE;
  IF PLOT THEN Schnittstelle_waehlen(1);   (* 2 bedeutet auf Datei plotten *)
                                           (* 1 ist parallele Schnittst    *)
                 (* 1 Einheit auf der X-Achse und Y-Achse entspricht .. cm *)
  Einh := 0.5;
  WI2D[1].X := -20.0;  WI2D[1].Y := -20.0;
                                               (* linker unterer Punkt     *)
                                               (* des Weltfensters         *)
  Ausgangszustand_definieren;                 (* Graphikgrundeinstellung   *)

  Eingabe;
  Dreiecks_Seiten;
  Winkeln;
  Seitenlaengen;
  Zeichnen;
  Beschriftung;
  WClosegraph;
END.